@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP           #-}
 {-# LANGUAGE DeriveGeneric #-}
 
 -----------------------------------------------------------------------------
@@ -103,7 +104,9 @@ import Control.Monad (liftM)
 import Distribution.Compat.Binary (Binary)
 import Data.List   ( sort )
 import Data.Char   ( isSpace, isAlpha )
+#if __GLASGOW_HASKELL__ < 710
 import Data.Monoid ( Monoid(..) )
+#endif
 import GHC.Generics (Generic)
 
 -- FIXME Not sure where this should live
@@ -263,7 +266,10 @@ instance Monoid GlobalFlags where
 -- * Config flags
 -- ------------------------------------------------------------
 
--- | Flags to @configure@ command
+-- | Flags to @configure@ command.
+--
+-- IMPORTANT: every time a new flag is added, 'D.C.Setup.filterConfigureFlags'
+-- should be updated.
 data ConfigFlags = ConfigFlags {
     --FIXME: the configPrograms is only here to pass info through to configure
     -- because the type of configure is constrained by the UserHooks.
@@ -455,7 +461,18 @@ configureOptions showOrParseArgs =
 
       ,option "" ["profiling"]
          "Executable profiling (requires library profiling)"
-         configProfExe (\v flags -> flags { configProfLib = v, configProfExe = v })
+         -- HACK: See #2409. Thankfully, this is 1.22-specific.
+         (\flags ->
+           case (configProfLib flags, configProfExe flags) of
+             (Flag a, Flag b)
+               | (a == b)
+               && ("cabalConfProf", "/TRUE") `elem` configProgramPaths flags
+                    -> configProfExe flags
+             _      -> NoFlag)
+         (\v flags -> flags
+               { configProfLib = v, configProfExe = v
+               , configProgramPaths = ("cabalConfProf", "/TRUE")
+               : configProgramPaths flags })
          (boolOpt [] [])
 
       ,option "" ["executable-profiling"]
@@ -1639,7 +1656,7 @@ instance Monoid ReplFlags where
 replCommand :: ProgramConfiguration -> CommandUI ReplFlags
 replCommand progConf = CommandUI
   { commandName         = "repl"
-  , commandSynopsis     = 
+  , commandSynopsis     =
       "Open an interpreter session for the given component."
   , commandDescription  = Just $ \pname -> wrapText $
          "If the current directory contains no package, ignores COMPONENT "
@@ -2082,7 +2099,6 @@ optionNumJobs get set =
         _        -> case reads s of
           [(n, "")]
             | n < 1     -> Left "The number of jobs should be 1 or more."
-            | n > 64    -> Left "You probably don't want that many jobs."
             | otherwise -> Right (Just n)
           _             -> Left "The jobs value should be a number or '$ncpus'"
 
